/*
 * Demo program 2 for kernel interrupt demo module
 * Author  : MHA@SSV <info@dilnetpc.com> <m.hasewinkel@web.de>
 * Date    : 27.08.2004
 * Version : 1.00
 * License : GPL
 */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "ssvirq.h"

/* we need a global variable, if NOT we oops */
volatile unsigned long irqcount = 0;

int main(void)
{
int device = 0, returnvalue = 0;

    /* get user identity */
    if (geteuid() != 0) {
	fprintf (stderr,"No root access rights !\n");
	exit(1);
    }
    /* delete possible directory or file with no error check */
    unlink("/dev/ssvirq");
    rmdir("/dev/ssvirq");
    /* make special file, like "mknod /dev/ssvirq c $SSVIRQ_MAJOR 0" */
    if (mknod("/dev/ssvirq", (0600 | S_IFCHR), (SSVIRQ_MAJOR << 8))) {
	perror("mknod");
	exit(1);
    }

    /* open ssvirq device driver */
    device = open("/dev/ssvirq", O_RDWR);
    if (device < 0) {
	perror("open");
	exit(1);
    }

    /* get current ssvirq driver "irqcount" */
    returnvalue = ioctl(device, SSVIRQ_GETVALUE, &irqcount);
    if (returnvalue < 0) {
	perror("ioctl SSVIRQ_GETVALUE");
	exit(-1);
    }

    /* tell ssvirq driver the address of "irqcount" */
    returnvalue = ioctl(device, SSVIRQ_SETPOINTER, &irqcount);
    if (returnvalue < 0) {
	perror("ioctl SSVIRQ_SETPOINTER");
	exit(1);
    }

    /* print endless "irqcount" - modified by ssvirq driver */
    for (;;) {
	printf("\rIRQ %d - irqcount %ld", SSVIRQ_IRQNUMBER, irqcount);
	fflush(stdout);
	usleep(1000);
    }

    /* close ssvirq device driver */
    close(device);

    return 0;
}
